"""
This type stub file was generated by pyright.
"""

from collections.abc import Sequence
from typing import TYPE_CHECKING

"""License specification constants."""
if TYPE_CHECKING:
    ...
NOT_SUPPORTED: set[str]
SPDX_LICENSE_MAP: dict[str, Sequence[str]]
SPDX_LICENSE_EXCEPTIONS: tuple[str]
